/*!@brief Einbinden der Fridolin Bibliothek */
#include "Fridolin.h"


/*!@brief Definieren fester Werte, um den Programmcode einfacher zu machen */
#define SCHWARZ 0
#define WEISS 1


/*!
 *@mainpage Fridolin LineFollower
 *@section Fridolin LineFollower Algorithmus
 *@author Christopher Bauer, Lukas Hemmerling, Änderungen durch die Bitschubser
 *@date   19.05.2019
 *@version  5.0
 * Implementierung des PID Reglungsalgorithmus
 * Der Code ist fuer Fridolin V1.3 vorgesehen
 * Bei V.1.4 muessen die Werte der Motoren-Pins vertauscht werden
 * (MRGeschwindigkeit = 5, MRRichtung = 3 und MLGeschwindigkeit = 9, MLRichtung = 6)
*****/


/*!
 *@remark Pins definieren und Objekte erzeugen
*/

using namespace STFU;

/*!@section Bodensensorobjekte erzeugen und Pinbelegung definieren */
static GroundSensor groundSensor = Fridolin::groundSensor();

/*!@section Motorobjekte erzeugen und Pinbelegung definieren */
static Engine engine = Fridolin::engine();


/*!@section Variablendeklaration */
/*!@brief Aktueller Wert des linken Bodensensors */
int bodenSensorLinks = 0;
/*!@brief Aktueller Wert des mittleren Bodensensors */
int bodenSensorMitte = 0;
/*!@brief Aktueller Wert des mittleren Bodensensors */
int bodenSensorRechts = 0;


void setup()
{}


void loop()
{
  /*!@brief Sensoren auslesen */
  bodenSensorLinks = groundSensor.left();
  bodenSensorMitte = !groundSensor.middle();
  bodenSensorRechts = groundSensor.right();
  
  
  /*!@brief Rechter Sensor erkennt die schwarze Linie */
  if (bodenSensorLinks==WEISS && bodenSensorRechts==SCHWARZ)
  {
  /*!@brief Fridolin dreht auf der Stelle nach rechts (50% der maximalen Geschwindigkeit) */
    engine.rotateRight(50_percent);
  }
  
  /*!@brief Linker Sensor erkennt die schwarze Linie */
  else if (bodenSensorLinks==SCHWARZ && bodenSensorRechts==WEISS)
  {
  /*!@brief Fridolin dreht auf der Stelle nach links (50% der maximalen Geschwindigkeit) */
    engine.rotateLeft(50_percent);
  }
  
  /*!@brief Aeussere Sensoren erkennen keine Linie, nur der mittlere Sensor */
  else if (bodenSensorLinks==WEISS && bodenSensorMitte==SCHWARZ && bodenSensorRechts==WEISS)
  {
  /*!@brief Fridolin faehrt vorwaerts (50% der maximalen Geschwindigkeit) */
    engine.driveForward(50_percent);
  }
  
  /*!@brief Beide aeussere Sensoren erkennen eine Schwarze Linie */
  else if (bodenSensorLinks==SCHWARZ && bodenSensorMitte==WEISS && bodenSensorRechts==SCHWARZ)
  {
  /*!@brief Fridolin bleibt stehen */
    engine.stop();
  }
}
