/*!@brief Einbinden der Fridolin Bibliothek */
#include "Fridolin.h"


/*!@brief Definieren fester Werte, um den Programmcode einfacher zu machen */
#define SCHWARZ 0
#define WEISS 1


/*!
 *@mainpage Fridolin LineFollower
 *@section Fridolin LineFollower Algorithmus
 *@author Christopher Bauer, Lukas Hemmerling, Änderungen durch die Bitschubser
 *@date   19.05.2019
 *@version  5.0
 * Implementierung des PID Reglungsalgorithmus
 * Der Code ist fuer Fridolin V1.3 vorgesehen
 * Bei V.1.4 muessen die Werte der Motoren-Pins vertauscht werden
 * (MRGeschwindigkeit = 5, MRRichtung = 3 und MLGeschwindigkeit = 9, MLRichtung = 6)
 *****/

/*!@brief Hier wird die letzte Fahrtrichtung gespeichert */
bool directionLeft = true;


/*!
 *@remark Pins definieren und Objekte erzeugen
*/

using namespace STFU;

/*!@section Bodensensorobjekte erzeugen und Pinbelegung definieren */
static GroundSensor groundSensor = Fridolin::groundSensor();

/*!@section Motorobjekte erzeugen und Pinbelegung definieren */
static Engine engine = Fridolin::engine();


/*!@section Variablendeklaration */
/*!@brief Aktueller Wert des linken Bodensensors */
int bodenSensorLinks = 0;
/*!@brief Aktueller Wert des mittleren Bodensensors */
int bodenSensorMitte = 0;
/*!@brief Aktueller Wert des mittleren Bodensensors */
int bodenSensorRechts = 0;


void setup()
{}


void loop()
{
  /*!@brief Sensoren auslesen */
  bodenSensorLinks = groundSensor.left();
  bodenSensorMitte = !groundSensor.middle();
  bodenSensorRechts = groundSensor.right();
  
  
  /*!@brief mittlere und rechter Sensor erkennt die schwarze Linie */
  if (bodenSensorLinks==WEISS && bodenSensorMitte==SCHWARZ && bodenSensorRechts==SCHWARZ)
  {
    engine.rotateRight(80_percent, 70_percent);
    directionLeft = false;
  }

  /*!@brief Rechter Sensor erkennt die schwarze Linie */
  else if (bodenSensorLinks==WEISS && bodenSensorMitte==WEISS && bodenSensorRechts==SCHWARZ)
  {
    engine.rotateRight(65_percent);
    directionLeft = false;
  }
  
  /*!@brief Linker und mittlere Sensor erkennt die schwarze Linie */
  else if (bodenSensorLinks==SCHWARZ && bodenSensorMitte==SCHWARZ && bodenSensorRechts==WEISS)
  {
    engine.rotateLeft(70_percent, 80_percent);
    directionLeft = true;
  }

  /*!@brief Linker Sensor erkennt die schwarze Linie */
  else if (bodenSensorLinks==SCHWARZ && bodenSensorMitte==WEISS && bodenSensorRechts==WEISS)
  {
    engine.rotateLeft(65_percent);
    directionLeft = true;
  }
  
  /*!@brief Aeussere Sensoren erkennen keine Linie, nur der mittlere Sensor */
  else if (bodenSensorLinks==WEISS && bodenSensorMitte==SCHWARZ && bodenSensorRechts==WEISS)
  {
    engine.driveForward(80_percent);
  }
  
  /*!@brief Beide aeussere Sensoren erkennen eine schwarze Linie und der mittlere nicht */
  else if (bodenSensorLinks==SCHWARZ && bodenSensorMitte==WEISS && bodenSensorRechts==SCHWARZ)
  {
    checkLastState();
  }
  
  /*!@brief Alle Sensoren erkennen eine weisse Linie */
  else if (bodenSensorLinks==WEISS && bodenSensorMitte==WEISS && bodenSensorRechts==WEISS)
  {
	/*!@brief Hier wird die letzte Fahrtrichtung ueberprueft und dementsprechend gehandelt */
    checkLastState();
  }
}

void checkLastState()
{
  if(directionLeft)
  {
	/*!@brief
	 * Wenn die letzte Fahrtrichtung Links war,<br>
	 * dreht Fridolin auf der Stelle nach Links
	 */
    engine.rotateLeft(70_percent);
  }
  else
  {
    /*!@brief
	 * Wenn die letzte Fahrtrichtung Rechts war,<br>
	 * dreht Fridolin auf der Stelle nach Rechts
	 */
    engine.rotateRight(70_percent);
  }
}
