#pragma once

#include "Button.h"


/*!
 * @author Die Bitschubser
 * @date 21.05.2019
 * @file Buttonbar.h
 * @version 3.1
 * @brief Enthaelt die Klasse Buttonbar
 *****/

namespace STFU
{
/*!
 * @class Buttonbar Buttonbar.h
 * @brief Klasse für die 3 Taster
 * Diese Klasse stellt die Initialisierung und Funktionen für die 3 Taster bereit
 *****/
class Buttonbar
{
public:
 /*!
  * @brief Der Defaultkonstruktor ist verboten
  */
  Buttonbar() = delete;
 /*!
  * @brief Buttonbar
  * @param[in] int left  Der Pin des erste Taster
  * @param[in] int mid  Der Pin des zweite Taster
  * @param[in] int right  Der Pin des dritte Taster
  */
  Buttonbar(int leftButtonPin, int midButtonPin, int rightButtonPin);
  /*!
   * @brief Im Destruktor werden die verwendeten Resourcen wieder freigegeben
   */
  ~Buttonbar();
  /*!
   * @brief Das System pausiert bis der linke Taster gedrueckt wurde
   */
  void waitForPushButtonLeft() const;
  /*!
   * @brief Das System pausiert bis der mittlere Taster gedrueckt wurde
   */
  void waitForPushButtonMid() const;
  /*!
   * @brief Das System pausiert bis der rechte Taster gedrueckt wurde
   */
  void waitForPushButtonRight() const;
  /*!
   * @brief Abfrage des Status von dem ersten/linken Taster
   * @return Liefert true, wenn der erste/linke Taster gedrueckt ist, sonst false
   */
  bool checkButtonLeft() const;
  /*!
   * @brief Abfrage des Status von dem zweiten/mittleren Taster
   * @return Liefert true, wenn der zweite/mittlere Taster gedrueckt ist, sonst false
   */
  bool checkButtonMid() const;
  /*!
   * @brief Abfrage des Status von dem dritten/rechten Taster
   * @return Liefert true, wenn der dritte/rechtenTaster gedrueckt ist, sonst false
   */
  bool checkButtonRight() const;

  /*!
   * @brief Anfrage nach dem ersten Taster
   * @return Liefert den ersten Taster
   */
  const Button& leftButton() const;
  /*!
   * @brief Anfrage nach dem zweiten Taster
   * @return Liefert den zweiten Taster
   */
  const Button& midButton() const;
  /*!
   * @brief Anfrage nach dem dritten Taster
   * @return Liefert den dritten Taster
   */
  const Button& rightButton() const;
private:
	/*! @brief Erster der drei Taster */
	Button* _leftButton = nullptr;
	/*! @brief Zweiter der drei Taster */
	Button* _midButton = nullptr;
	/*! @brief Dritter der drei Taster */
	Button* _rightButton = nullptr;
};

}