#pragma once

#include <Wire.h>
#include <LiquidCrystal_I2C.h>

/*!
 * @author Die Bitschubser - Kevin Staut
 * @date 21.05.2019
 * @file Display.h
 * @version 2.0
 * @brief Enthaelt die Klasse Display <br>
 * Es muss die externe Library "New-LiquidCrystal-master eingebunden werden"
 *****/

namespace STFU
{
/*!
 * @class Display Display.h
 * @brief Diese Klasse stellt die Initialisierung und Funktionen für das Display bereit
 *****/
class Display
{
public:
   /*!
    * @brief Der Defaultkonstruktor ist verboten
    */
	Display() = delete;
    /*!
    * @brief Dieser Konstruktor initialisiert das Objekt und legt die entsprechenden Pins fest
    */
	Display(uint8_t lcd_Addr, uint8_t En, uint8_t Rw, uint8_t Rs,
            uint8_t d4, uint8_t d5, uint8_t d6, uint8_t d7,
			uint8_t background);
   /*!
    * @brief Im Destruktor werden die verwendeten Resourcen wieder freigegeben  
    */
    ~Display();

   /*!
    * @brief Einen Text in Zeile 1 des Displays anzeigen.
    * @param[in] String  Text der angezeigt werden soll.
    */
    void textRowOne(String text);
   /*!
    * @brief Einen Text in Zeile 2 des Displays anzeigen.
    * @param[in] String  Text der angezeigt werden soll.
    */
    void textRowTwo(String text);
   /*!
    * @brief Einen Text in Zeile 1 und Zeile 2 des Displays anzeigen.
    * @param[in] String textrowone  Text der in Zeile 1 gezeigt wird.
    * @param[in] String textrowtow  Text der in Zeile 2 gezeigt wird.
    */
    void textInBothRows(String textrowone, String textrowtwo);
   /*!
    * @brief Diese Funktion löscht den Inhalt des Displays und setzt es zurück.
    */
    void clear();
   /*!
    * @brief Diese Funktion muss in der Setup Funktion aufgerufen werden, damit das Display verwendet werden kann.
    */
	void setup();

private:
	/*!@brief Instanz der externen LiquidCrystal_I2C Klasse / Bibliothek */
    LiquidCrystal_I2C* _LCD = nullptr;
};

}