#include "Engine.h"

namespace STFU
{

/*! LITERAL */
#define _100_PERCENT 100
#define _0_PERCENT 0
int operator""_percent(unsigned long long int p)
{
  if(p >= _100_PERCENT)
  {
    return Engine::MAX_SPEED;
  }
  else if(p <= _0_PERCENT)
  {
    return Engine::MIN_SPEED;
  }
  else
    return (static_cast<int>(Engine::MAX_SPEED * p) / 100);
}
#undef _0_PERCENT
#undef _100_PERCENT


Engine::Engine(Motor& left, Motor& right)
  : _leftMotor(&left),
    _rightMotor(&right)
{}


Engine::~Engine()
{
  _leftMotor = nullptr;
  _rightMotor = nullptr;
}



int Engine::speed() const
{
  return average(_leftSpeed, _rightSpeed); // hier wird der Durchschnitt ermittelt
}


int Engine::direction() const
{
  int rL = _leftMotor->direction();
  int rR = _rightMotor->direction();
  return average(rL, rR); // hier wird der Durchschnitt ermittelt
}


void Engine::stop()
{
  speed(Engine::MIN_SPEED);
  direction(FORWARD);
}

void Engine::driveForward(int s)
{
  driveForward(s, s);
}

void Engine::driveForward(int sL, int sR)
{
  direction(FORWARD);
  speed(sL, sR);
}

void Engine::driveBackward(int s)
{
  direction(BACKWARD);
  speed(backward(s));
}

void Engine::driveBackward(int sL, int sR)
{
  direction(BACKWARD);
  speed(backward(sL), backward(sR));
}


void Engine::rotateLeft(int s)
{
  rotateLeft(s, s);
}

void Engine::rotateLeft(int sL, int sR)
{
  direction(BACKWARD, FORWARD);
  speed(backward(sL), sR);
}

void Engine::turnLeft(int sR)
{
  int sL = static_cast<int>(sR / 2);
  driveForward(sL, sR);
}


void Engine::rotateRight(int s)
{
  rotateRight(s, s);
}

void Engine::rotateRight(int sL, int sR)
{
  direction(FORWARD, BACKWARD);
  speed(sL, backward(sR));
}

void Engine::turnRight(int sL)
{
  int sR = static_cast<int>(sL / 2);
  driveForward(sL, sR);
}

const Motor& Engine::leftMotor() const
{
  return (*_leftMotor);
}

const Motor& Engine::rightMotor() const
{
  return (*_rightMotor);
}




/* PRIVATE */
void Engine::speed(int s)
{
  speed(s, s);
}

void Engine::speed(int sL, int sR)
{
  _leftSpeed = sL;
  _rightSpeed = sR;
  _leftMotor->speed(sL);
  _rightMotor->speed(sR);
}

void Engine::direction(int d)
{
  direction(d, d);
}

void Engine::direction(int dL, int dR)
{
  _leftMotor->direction(dL);
  _rightMotor->direction(dR);
}


int Engine::average(int a, int b) const
{
	return static_cast<int>((a + b) / 2);
}

int Engine::backward(int s) const
{
	return Engine::MAX_SPEED - s;
}

}