#pragma once

#include "Arduino.h"


/*!
 * @author Die Bitschubser
 * @date 20.05.2019
 * @file Motor.h
 * @version 2.0
 * @brief Enthaelt die Klasse Motor
 *****/

namespace STFU
{
/*!
 * @class Motor Motor.h
 * @brief Diese Klasse stellt die Initialisierung und Funktionen für Motoren bereit
 *****/
class Motor
{
public:
  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  Motor() = delete;
  /*!
   * @brief Konstruktor mit Pinbelegung fuer Geschwindigkeit und Richtung
   * @param[in] int pinSpeed  Pin fuer die Motorgeschwindigkeit
   * @param[in] int pinDirection  Pin fuer die Motorrichtung
   */
  Motor(int pinSpeed, int pinDirection);

  /*!
   * @brief Destruktor ist default
   */
  ~Motor() = default;

  /*!
   * @brief Regelt die Geschwindigkeit des Motorobjekts
   * @param[in] int  Geschwindigkeit die auf den Motor uebertragen werden soll
   */
  void speed(int);
  /*!
   * @brief Abfrage der Motorgeschwindigkeit
   * @return Gibt die analoge Geschwindigkeit des Motorobjekts zurueck
   */
  int speed() const;

  /*!
   * @brief Regelt die Richtung des Motorobjekts
   * @param[in] int  Richtung in die der Motor drehen soll (255 - rueckwaerts, 0 - vorwaerts)
   */
  void direction(int);
  /*!
   * @brief Abfrage der Motorrichtung
   * @return Gibt die aktuelle Richtung des Motorobjekts zurueck (0 - 255)
   */
  int direction() const;

private:
  /*! @brief Pin fuer die Geschwindigkeitsregelung des Motors */
  const int _pinSpeed;
  /*! @brief Pin fuer die Richtungsregelung des Motors */
  const int _pinDirection;
};

}