#include "Fridolin.h"

namespace STFU
{
/*!@section Antrieb erzeugen Pinbelegung definieren */

/*!@subsection Motorsensor Digital links */
/*!@brief ZuweisungDigitalPin D5 fuer Geschwindigkeit */
#define MotorLeftSpeed 5
/*!@brief Zuweisung Digital Pin D3 fuer Richtung */
#define MotorLeftDirection 3

/*!@subsection Motorsensor Digital rechts */
/*!@brief Zuweisung Digital Pin D9 fuer Geschwindigkeit */
#define MotorRightSpeed 9
/*!@brief Zuweisung Digital Pin D6 fuer Richtung */
#define MotorRightDirection 6

/*!@brief Instanzen der Motor Klasse */
static Motor left{ MotorLeftSpeed, MotorLeftDirection };
static Motor right{ MotorRightSpeed, MotorRightDirection };

/*!@brief Instanz der Engine Klasse */
static Engine _engine{left, right};

const Engine& Fridolin::engine()
{
    return _engine;
}

#undef MotorRightDirection
#undef MotorRightSpeed
#undef MotorLeftDirection
#undef MotorLeftSpeed



/*!@section Bodensensorobjekt erzeugen und Pinbelegung definieren */

/*!@subsection Bodensensor Digital links */
/*!@brief Zuweisung Digital Pin D13 fuer Sensor Links --> schwarz = 0 */
#define BSLinks 13

/*!@subsection Bodensensor Digital rechts */
/*!@brief Zuweisung Digital Pin D11 fuer Sensor Rechts --> schwarz = 0 */
#define BSRechts 11

/*!@subsection Bodensensor Analog mitte */
/*!@brief Bodensensor mitte A3   --> 0-1023 --> 0 schwarz, 1023 hell */
#define BSMitte A3

/*!@brief Instanzen der GroundSensor Klasse */
static DigitalSensor sensorLeft{ BSLinks };
static DigitalSensor sensorRight{ BSRechts };
static AnalogSensor sensorMiddle{ BSMitte };

/*!@brief Instanz der GroundSensor Klasse */
static GroundSensor _groundSensor{sensorLeft, sensorRight, sensorMiddle};

const GroundSensor& Fridolin::groundSensor()
{
    return _groundSensor;
}

#undef BSMitte
#undef BSRechts
#undef BSLinks



/*!@subsection Ultraschallsensorobjekt erzeugen und Pinbelegung definieren */
/*!@brief Zuweisung 10 fuer Servo-Pin */
#define Servopin 10
/*!@brief Zuweisung 7 fuer Trigger-Pin */
#define Triggerpin 7
/*!@brief Zuweisung 8 fuer Echo-Pin */
#define Echopin 8

/*!@brief Instanzen der Ultraschallsensor Klasse */
static UltrasonicSensor _ultrasonicsensor {Servopin, Triggerpin, Echopin};

const UltrasonicSensor& Fridolin::ultrasonicSensor()
{
    return _ultrasonicsensor;
}

#undef Echopin
#undef Triggerpin
#undef Servopin


/*!@subsection Abstandssensorenobjekt erzeugen und Pinbelegung definieren */
/*!@brief Zuweisung A2 fuer linken Abstandssensor */
#define SL A2
/*!@brief Zuweisung A1 fuer mittleren Abstandssensor */
#define SM A1
/*!@brief Zuweisung A0 fuer rechten Abstandssensor */
#define SR A0

static FrontSensor _frontsensor {SL, SM, SR};

const FrontSensor& Fridolin::frontSensor()
{
    return _frontsensor;
}

#undef SR
#undef SM
#undef SL



/*!@section Tasterobjekt erzeugen und Pinbelegung definieren */
/*!@brief Zuweisung Eingangspin D12 fuer Taster Nr. 1 */
#define buttonPin1 12
/*!@brief Zuweisung Eingangspin D2 fuer Taster Nr. 2 */
#define buttonPin2 2
/*!@brief Zuweisung Eingangspin D4 fuer Taster Nr. 3 */
#define buttonPin3 4

/*!@brief Instanz der Buttonbar Klasse */
static Buttonbar _buttonbar{ buttonPin1, buttonPin2, buttonPin3 };

const Buttonbar& Fridolin::buttonbar()
{
	return _buttonbar;
}

#undef buttonPin3
#undef buttonPin2
#undef buttonPin1



#ifdef FLED
/*!@section Ledobjekt erzeugen und Pinbelegung definieren */
#define ledPin 13

/*!@brief Instanz der LED Klasse */
static LED _led{ ledPin };

const LED& Fridolin::led()
{
	return _led;
}

#undef ledPin
#endif



/*!@subsection Displayobjekt erzeugen und Pinbelegung definieren */
/*!@brief Zuweisung 0x3F fuer Adresse I2C Bus (TWI) */
#define I2C_ADDR 0x3F
/*!@brief Zuweisung 3 fuer Hintergrund */
#define Hintergrund 3
/*!@brief Zuweisung 2 fuer En Pin */
#define En_pin 2
/*!@brief Zuweisung 1 fuer Rw Pin */
#define Rw_pin 1
/*!@brief Zuweisung 0 fuer Rs Pin */
#define Rs_pin 0
/*!@brief Zuweisung D4 */
#define D4_pin 4
/*!@brief Zuweisung D5 */
#define D5_pin 5
/*!@brief Zuweisung D6 */
#define D6_pin 6
/*!@brief Zuweisung D7 */
#define D7_pin 7

/*!@brief Instanzen der Display Klasse */
static Display _display{ I2C_ADDR, En_pin, Rw_pin, Rs_pin, D4_pin, D5_pin, D6_pin, D7_pin, Hintergrund };

const Display& Fridolin::display()
{
    return _display;
}

#undef D7_pin
#undef D6_pin
#undef D5_pin
#undef D4_pin
#undef Rs_pin
#undef Rw_pin
#undef En_pin
#undef Hintergrund
#undef I2C_ADDR



/*!@subsection Ultraschallsensorobjekt erzeugen und Pinbelegung definieren */
/*!@brief Zuweisung A6 fuer Batterie-Pin */
#define Battery A6
void Fridolin::batteryStatus() 
{
    _display.clear();
    float messen = analogRead(Battery);
    float batt = (messen*147)/47*5/1023;
    String akkustand;

    if(batt < 9)
	{
        akkustand = ":-(";
    }
	else if(batt < 10)
	{
        akkustand = ":-/";
    }
	else
	{
        akkustand = ":-)";
    }

	String akku{ "Akkustand:  " + akkustand };
	String battString{ static_cast<String>(batt) + " Volt" };
    if(batt < 9.99)
	{
        _display.textInBothRows(akku, " " + battString);
    }
	else
	{
        _display.textInBothRows(akku, battString);
    }
}
#undef battery



void Fridolin::setup()
{
	/* Display setup */
	_display.setup();
}

}