#pragma once

/*!@brief Einbinden aller Komponenten des Fridolins */
#include "Engine/Engine.h"
#include "Sensor/GroundSensor.h"
#include "Sensor/UltrasonicSensor.h"
#include "Sensor/FrontSensor.h"
#include "Display/Display.h"
#include "Button/Buttonbar.h"
#include "LED/LED.h"

/*!
 * @author Die Bitschubser
 * @date 10.06.2019
 * @file Fridolin.h
 * @version 3.1
 * @brief Enthaelt die Klasse Fridolin
 *****/

/*!@brief Super Tolle Fridolin Utilities<br>
 * Alle Klassen aus dieser Bibliothek haben den Präfix: STFU
 */
namespace STFU
{
/*!
 * @class Fridolin Fridolin.h
 * @brief Diese Klasse stellt die Initialisierung und Funktionen für den Fridolin bereit
 *****/
class Fridolin
{
public:
   /*!
    * @brief Der Defaultkonstruktor ist verboten
    */
    Fridolin() = delete;
   /*!
	* @brief Abfrage des Antriebs von Fridolin
	* @return Der Fridolin-Antrieb
	*/
    static const Engine& engine();
   /*!
	* @brief Abfrage des BodenSensors von Fridolin
	* @return Der Fridolin-BodenSensors
	*/
    static const GroundSensor& groundSensor();
   /*!
	* @brief Abfrage des Ultraschallsensors von Fridolin
	* @return Der Fridolin-Ultraschallsensor
	*/
    static const UltrasonicSensor& ultrasonicSensor();
   /*!
	* @brief Abfrage der Abstandssensoren von Fridolin
	* @return Der Fridolin-Abstandssensoren
	*/
    static const FrontSensor& frontSensor();
   /*!
	* @brief Abfrage der Buttonbar von Fridolin
	* @return Die Fridolin-Buttonbar
	*/
	static const Buttonbar& buttonbar();
   /*!
	* @brief Abfrage der LED von Fridolin
	* @return Die Fridolin-LED
	*/
	static const LED& led();
   /*!
	* @brief Abfrage des Displays von Fridolin
	* @return Das Fridolin-Display
	*/
    static const Display& display();


   /*!
	* @brief Abfrage des Akkustands von Fridolin<br>
	* Der Akkustand wird im Display angezeigt.
	*/
    static void batteryStatus();

   /*!
	* @brief Die Setup-Funktion von Fridolin<br>
	* Diese Funktion muss in der Setup-Funktion aufgerufen werden
	*/
	static void setup();
};

}