#pragma once

#include "Sensor.h"


/*!
 * @author Die Bitschubser - Felix Mayer
 * @date 09.06.2019
 * @file FrontSensor.h
 * @version 1.1
 * @brief Enthaelt die Klasse FrontSensor
 *****/

namespace STFU
{
/*!
 * @class FrontSensor FrontSensor.h
 * @brief Klasse für die 3 Infrarotsensoren
 * Diese Klasse stellt die Initialisierung und Funktionen für die 3 Infrarotsensoren bereit
 *****/
class FrontSensor
{
public:
  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  FrontSensor() = delete;
  /*!
   * @brief FrontSensor
   * @param[in] int left  Der linke Infrarotsensor Pin
   * @param[in] int mid  Der mittlere Infrarotsensor Pin
   * @param[in] int right  Der rechte Infrarotsensor Pin
   */
  FrontSensor(int leftPin, int midPin, int rightPin);
  /*!
   * @brief Im Destruktor werden die verwendeten Resourcen wieder freigegeben
   */
  ~FrontSensor();

  /*!
   * @brief Abfrage der messbaren Distanz des linken Sensors
   * @return Liefert int, welches die Distanz in cm darstellt
   */
  int leftFrontDistance() const;
  /*!
   * @brief Abfrage der messbaren Distanz des mittleren Sensors
   * @return Liefert int, welches die Distanz in cm darstellt
   */
  int midFrontDistance() const;
  /*!
   * @brief Abfrage der messbaren Distanz des rechten Sensors
   * @return Liefert int, welches die Distanz in cm darstellt
   */
  int rightFrontDistance() const;

  /*!
   * @brief Analoge Abfrage des Linken Infrarotsensors
   * @return Gibt den aktuellen analogen Wert des linken Infrarotsensors zurueck.
   */
  int leftValue() const;
  /*!
   * @brief Analoge Abfrage des Mittleren Infrarotsensors
   * @return Gibt den aktuellen analogen Wert des mittleren Infrarotsensors zurueck.
   */
  int midValue() const;
  /*!
   * @brief Analoge Abfrage des Rechten Infrarotsensors
   * @return Gibt den aktuellen analogen Wert des rechten Infrarotsensors zurueck.
   */
  int rightValue() const;


private:
  /*! @brief Linker der drei Infrarotsensoren */
  InfraredSensor* _leftIR = nullptr;
  /*! @brief Mittlerer der drei Infrarotsensoren */
  InfraredSensor* _midIR = nullptr;
  /*! @brief Rechter der drei Infrarotsensoren */
  InfraredSensor* _rightIR = nullptr;
};

}