#pragma once

#include "Arduino.h"


/*!
 * @author Die Bitschubser
 * @date 09.06.2019
 * @file Sensor.h
 * @version 2.2
 * @brief Enthaelt die Klassen Sensor, DigitalSensor, AnalogSensor
 *****/

namespace STFU
{
/*!
 * @class Sensor Sensor.h
 * @brief Basisklasse als Interface für die Sensoren
 *****/
class Sensor
{
public:
  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  Sensor() = delete;
  /*!
   * @brief Konstruktor mit Pinbelegung fuer Messung
   * @param[in] int  Pin fuer den Sensor
   */
  explicit Sensor(int s);

  /*!
   * @brief Der Destruktor ist default
   */
  virtual ~Sensor() = default;

  /*!
   * @brief Abfrage des Sensorwertes
   * @return Gibt den aktuellen Wert des Sensors zurueck
   */
  // return type überladen geht nicht, andere lösung?
  //virtual bool read() const;

protected:
  /*! @brief Pin fuer den Sensor*/
  const int _pin;
};



/*!
 * @class DigitalSensor Sensor.h
 * @brief Abgelteitete Sensorenklasse für digitale Sensoren
 *****/
class DigitalSensor : Sensor
{
public:
  /*!
   * @brief Konstruktor mit Pinbelegung fuer Messung
   * @param[in] int  Pin fuer den digitalen Sensor
   */
  explicit DigitalSensor(int);

  /*!
   * @brief Abfrage des Sensorwertes
   * @return Gibt den aktuellen digitalen Wert des Sensors zurueck
   */
  bool read() const;
};


/*!
 * @class AnalogSensor Sensor.h
 * @brief Abgelteitete Sensorenklasse für analoge Sensoren
 *****/
class AnalogSensor : Sensor
{
public:
  /*! @brief Maximaler Sensorwert */
  static const int MAX_VALUE = 1023;
  /*! @brief Minimaler Sensorwert */
  static const int MIN_VALUE = 0;

  /*!
   * @brief Konstruktor mit Pinbelegung fuer Messung
   * @param[in] int  Pin fuer den analogen Sensor
   */
  explicit AnalogSensor(int);

  /*!
   * @brief Abfrage des Sensorwertes
   * @return Gibt den aktuellen analogen Wert des Sensors zurueck
   */
  int read() const;
};


/*!
 * @class InfraredSensor Sensor.h
 * @brief Abgelteitete Sensorenklasse für Infrarotsensoren
 *****/
class InfraredSensor final : public AnalogSensor
{
public:
  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  InfraredSensor() = delete;
  /*!
   * @brief Konstruktor mit Pinbelegung fuer Messung
   * @param[in] int  Stellt den angepeilten PIN für den Infrarotsensor dar
   */
  explicit InfraredSensor(int);

  /*!
   * @brief Abfrage der messbaren Distanz des Sensors
   * @return Liefert int, welches die Distanz in cm darstellt
   */
  int distance() const;
};

}