#pragma once


#include "Servo.h"
#include "Arduino.h"


/*!
 * @author Die Bitschubser - Dominik Schramm
 * @date 04.06.2019
 * @file UltrasonicSensor.h
 * @version 1.0
 * @brief Enthaelt die Klasse UltrasonicSensor
 *****/

namespace STFU
{
/*!
 * @class UltrasonicSensor UltrasonicSensor.h
 * @brief Diese Klasse stellt die Initialisierung und Funktionen für den Ultraschall-Senor mit Servo bereit
 *****/
class UltrasonicSensor
{
public:
 /*! @brief Maximale Drehzahl des Servos */
  static const int MAX_SERVO_DEGREE = 180;
  /*! @brief Minimale Drehzahl des Servos */
  static const int MIN_SERVO_DEGREE = 0;
  
  /*!
   * @brief Der Defaultkonstruktor ist verboten
   */
  UltrasonicSensor() = delete;
  
  /*!
  * @brief UltrasonicSensor Konstruktor
  * @param[in] int servo  Pin des Servos
  * @param[in] int trigger  Pin des Trigger vom Ultraschallsensors
  * @param[in] int echo  Pin des Echo vom Ultraschallsensors
  */
  UltrasonicSensor(int,int,int);
  
   /*!
  * @brief Abfrage der Entfernung des Ultraschallsensors
  * @return Gibt die Entfernung zurück
  */
  int read() const;
  
   /*!
  * @brief Abfrage der Entfernung des Ultraschallsensors mit vorheriger Einstellung des Servos
  * @return Gibt die Entfernung zurück
  */
  int read(int);
  
   /*!
  * @brief Abfrage, ob eine Wand da ist
  * @return Gibt zurück, ob eine Wand unter 30 cm da ist
  */
  bool readDigital() const;
  
   /*!
  * @brief Abfrage, ob eine Wand da ist mit vorheriger Einstellung des Servos
  * @return Gibt zurück, ob eine Wand unter 30 cm da ist
  */
  bool readDigital(int);
  
   /*!
  * @brief Setzen der Richtung des Servos
  * @return Gibt die dauer des drehens zurück
  */
  int servo(int);
  
   /*!
  * @brief Aktuelle Gradzahl
  * @return Gibt die aktuelle Gradzahl zurück
  */
  int servo() const;
 
   /*!
  * @brief Kalibrieren des Servos
  * @param[in] int degree_middle  Ausrichtung in die Mitte
  * @param[in] int degree_right  Ausrichtung nach Rechts
  */
  void setup(int, int);

private:
  /*! @brief Servoausrichtung nach vorne */
  int _degreeMiddle=90;
  /*! @brief Servoausrichtung nach rechts */
  int _degreeRight=180;
  /*! @brief Aktuelle Ausrichtung des Servos */
  int _degreeServo=90;

  /*! @brief Ultraschallsensor Trigger-Pin */
  int _triggerpin;
  /*! @brief Ultraschallsensor Echo-Pin */
  int _echopin;
  /*! @brief Servo-Pin */
  int _servopin;
  /*! @brief Servo-Object */
  Servo _servo;
};

}